"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Binding = void 0;
const events_1 = require("events");
class Binding extends events_1.EventEmitter {
    constructor() {
        super();
        this.is_connected = false;
        this.ssid = '0';
        this.list_promise = [];
        process.stdin.on('data', (chunk) => {
            try {
                let data = JSON.parse(chunk.toString());
                if (data.try_connect == 'connected') {
                    this.is_connected = true;
                    Binding.sendUpdateInternal({
                        ping: true
                    });
                    setInterval(function () {
                        Binding.sendUpdateInternal({
                            ping: true
                        });
                    }, 10000);
                    this.emit('connect', data.user_id);
                }
                else if (data.ping_with_response) {
                    Binding.sendUpdateInternal({
                        ping_with_response: true
                    });
                }
                else if (data.ssid == this.ssid) {
                    if (data.uid !== undefined) {
                        if (this.list_promise[data.uid] !== undefined) {
                            if (data.data !== undefined) {
                                this.list_promise[data.uid](data.data);
                            }
                            else {
                                this.list_promise[data.uid](null);
                            }
                        }
                    }
                    else {
                        this.emit('request', data.data);
                    }
                }
            }
            catch (e) { }
        });
        this.ssid = Binding.makeID(12);
        Binding.sendUpdateInternal({
            try_connect: this.ssid
        });
    }
    async sendUpdate(update) {
        if (this.is_connected) {
            let uid = Binding.makeID(12);
            Binding.sendUpdateInternal({
                data: update,
                uid: uid,
                ssid: this.ssid
            });
            return new Promise((resolve) => {
                this.list_promise[uid] = (data) => {
                    resolve(data);
                    delete this.list_promise[uid];
                };
            });
        }
        else {
            throw new Error('No connected client');
        }
    }
    static log(message, verbose_mode) {
        Binding.sendUpdateInternal({
            log_message: message,
            verbose_mode: verbose_mode,
        });
    }
    static makeID(length) {
        let result = '';
        let characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        let charactersLength = characters.length;
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
        }
        return result;
    }
    static sendUpdateInternal(update) {
        console.log(JSON.stringify(update));
    }
}
exports.Binding = Binding;
Binding.DEBUG = 1;
Binding.INFO = 2;
Binding.WARNING = 3;
Binding.ERROR = 4;
